using Gebhardt.StoreWare.Wcs.HostBooking.InterfaceWcsWms;

namespace Gebhardt.StoreWare.Wcs.HostBooking
{

	public class LeNoMissingException : FromWmsException
	{
		public LeNoMissingException(string message) : base(message) { }
	}

	public class LeNoWrongFormatException : FromWmsException
	{
		public LeNoWrongFormatException(string message) : base(message) { }
	}

	public class LeNoTooShortException : FromWmsException
	{
		public LeNoTooShortException(string message) : base(message) { }
	}

	public class LeNoTooLongException : FromWmsException
	{
		public LeNoTooLongException(string message) : base(message) { }
	}

	public class TransportDestinationInvalidException : FromWmsException
	{
		public TransportDestinationInvalidException(string message) : base(message) { }
	}

	public class LeAlreadyInStorageException : FromWmsException
	{
		public LeAlreadyInStorageException(string message) : base(message) { }
	}

	public class LeNotReachableException : FromWmsException
	{
		public LeNotReachableException(string message) : base(message) { }
	}

	public class LeHasActiveTransportOrderException : FromWmsException
	{
		public LeHasActiveTransportOrderException(string message) : base(message) { }
	}
}
